# This file is generated. Do not modify by hand.
from enum import Enum


class CommandCode(Enum):
    """
    Named constants for all Zaber Binary protocol commands.
    """

    RESET = 0

    HOME = 1

    RENUMBER = 2

    STORE_CURRENT_POSITION = 16

    RETURN_STORED_POSITION = 17

    MOVE_TO_STORED_POSITION = 18

    MOVE_ABSOLUTE = 20

    MOVE_RELATIVE = 21

    MOVE_AT_CONSTANT_SPEED = 22

    STOP = 23

    SET_ACTIVE_AXIS = 25

    SET_AXIS_DEVICE_NUMBER = 26

    SET_AXIS_INVERSION = 27

    SET_AXIS_VELOCITY_PROFILE = 28

    SET_AXIS_VELOCITY_SCALE = 29

    LOAD_EVENT_INSTRUCTION = 30

    RETURN_EVENT_INSTRUCTION = 31

    SET_CALIBRATION_MODE = 33

    SET_JOYSTICK_CALIBRATION_MODE = 33

    READ_OR_WRITE_MEMORY = 35

    RESTORE_SETTINGS = 36

    SET_MICROSTEP_RESOLUTION = 37

    SET_RUNNING_CURRENT = 38

    SET_HOLD_CURRENT = 39

    SET_DEVICE_MODE = 40

    SET_HOME_SPEED = 41

    SET_START_SPEED = 41

    SET_TARGET_SPEED = 42

    SET_ACCELERATION = 43

    SET_MAXIMUM_POSITION = 44

    SET_CURRENT_POSITION = 45

    SET_MAXIMUM_RELATIVE_MOVE = 46

    SET_HOME_OFFSET = 47

    SET_ALIAS_NUMBER = 48

    SET_LOCK_STATE = 49

    RETURN_DEVICE_ID = 50

    RETURN_FIRMWARE_VERSION = 51

    RETURN_POWER_SUPPLY_VOLTAGE = 52

    RETURN_SETTING = 53

    RETURN_STATUS = 54

    ECHO_DATA = 55

    RETURN_FIRMWARE_BUILD = 56

    RETURN_CURRENT_POSITION = 60

    RETURN_SERIAL_NUMBER = 63

    SET_PARK_STATE = 65

    SET_PERIPHERAL_ID = 66

    RETURN_DIGITAL_INPUT_COUNT = 67

    READ_DIGITAL_INPUT = 68

    READ_ALL_DIGITAL_INPUTS = 69

    RETURN_DIGITAL_OUTPUT_COUNT = 70

    READ_DIGITAL_OUTPUT = 71

    READ_ALL_DIGITAL_OUTPUTS = 72

    WRITE_DIGITAL_OUTPUT = 73

    WRITE_ALL_DIGITAL_OUTPUTS = 74

    RETURN_ANALOG_INPUT_COUNT = 75

    READ_ANALOG_INPUT = 76

    RETURN_ANALOG_OUTPUT_COUNT = 77

    MOVE_INDEX = 78

    SET_INDEX_DISTANCE = 79

    SET_CYCLE_DISTANCE = 80

    SET_FILTER_HOLDER_ID = 81

    RETURN_ENCODER_COUNT = 82

    RETURN_CALIBRATED_ENCODER_COUNT = 83

    RETURN_CALIBRATION_TYPE = 84

    RETURN_CALIBRATION_ERROR = 85

    RETURN_PERIPHERAL_SERIAL_NUMBER = 86

    SET_PERIPHERAL_SERIAL_NUMBER = 86

    FORCE_ABSOLUTE = 87

    FORCE_OFF = 88

    RETURN_ENCODER_POSITION = 89

    RETURN_PERIPHERAL_ID_PENDING = 91

    RETURN_PERIPHERAL_SERIAL_PENDING = 92

    ACTIVATE = 93

    SET_AUTO_REPLY_DISABLED_MODE = 101

    SET_MESSAGE_ID_MODE = 102

    SET_HOME_STATUS = 103

    SET_HOME_SENSOR_TYPE = 104

    SET_AUTO_HOME_DISABLED_MODE = 105

    SET_MINIMUM_POSITION = 106

    SET_KNOB_DISABLED_MODE = 107

    SET_KNOB_DIRECTION = 108

    SET_KNOB_MOVEMENT_MODE = 109

    SET_KNOB_JOG_SIZE = 110

    SET_KNOB_VELOCITY_SCALE = 111

    SET_KNOB_VELOCITY_PROFILE = 112

    SET_ACCELERATION_ONLY = 113

    SET_DECELERATION_ONLY = 114

    SET_MOVE_TRACKING_MODE = 115

    SET_MANUAL_MOVE_TRACKING_DISABLED_MODE = 116

    SET_MOVE_TRACKING_PERIOD = 117

    SET_CLOSED_LOOP_MODE = 118

    SET_SLIP_TRACKING_PERIOD = 119

    SET_STALL_TIMEOUT = 120

    SET_DEVICE_DIRECTION = 121

    SET_BAUD_RATE = 122

    SET_PROTOCOL = 123

    CONVERT_TO_ASCII = 124



# This file is generated. Do not modify by hand.
from enum import Enum


class TriggerAction(Enum):
    """
    Trigger action identifier.
    """

    ALL = 0
    """Refers to all trigger actions."""

    A = 1
    """Action A on a trigger."""

    B = 2
    """Action B on a trigger."""



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict, List
from collections.abc import Iterable
import zaber_bson
from .device_db_inner_error import DeviceDbInnerError


@dataclass
class DeviceDbFailedExceptionData:
    """
    Contains additional data for a DeviceDbFailedException.
    """

    code: str
    """
    Code describing type of the error.
    """

    inner_errors: List[DeviceDbInnerError]
    """
    A list of errors that occurred while trying to access information from the device database.
    """

    @staticmethod
    def zero_values() -> 'DeviceDbFailedExceptionData':
        return DeviceDbFailedExceptionData(
            code="",
            inner_errors=[],
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'DeviceDbFailedExceptionData':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return DeviceDbFailedExceptionData.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'code': str(self.code or ''),
            'innerErrors': [item.to_dict() for item in self.inner_errors] if self.inner_errors is not None else [],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'DeviceDbFailedExceptionData':
        return DeviceDbFailedExceptionData(
            code=data.get('code'),  # type: ignore
            inner_errors=[DeviceDbInnerError.from_dict(item) for item in data.get('innerErrors')],  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.code is not None:
            if not isinstance(self.code, str):
                raise ValueError(f'Property "Code" of "DeviceDbFailedExceptionData" is not a string.')

        if self.inner_errors is not None:
            if not isinstance(self.inner_errors, Iterable):
                raise ValueError('Property "InnerErrors" of "DeviceDbFailedExceptionData" is not iterable.')

            for i, inner_errors_item in enumerate(self.inner_errors):
                if inner_errors_item is None:
                    raise ValueError(f'Item {i} in property "InnerErrors" of "DeviceDbFailedExceptionData" is None.')

                if not isinstance(inner_errors_item, DeviceDbInnerError):
                    raise ValueError(f'Item {i} in property "InnerErrors" of "DeviceDbFailedExceptionData" is not an instance of "DeviceDbInnerError".')

                inner_errors_item.validate()



# This file is generated. Do not modify by hand.
from enum import Enum


class CyclicDirection(Enum):
    """
    Direction a cyclic device should move when doing an absolute move.
    """

    POSITIVE = 0
    """Move to the target in a positive direction."""

    NEGATIVE = 1
    """Move to the target in a negative direction."""

    SHORTEST = 2
    """Take the shortest path to the target position."""



# This file is generated. Do not modify by hand.
from enum import Enum


class AxisType(Enum):
    """
    Denotes type of an axis and units it accepts.
    """

    UNKNOWN = 0
    """Axis type could not be determined."""

    LINEAR = 1
    """A linear axis that accepts length units for position."""

    ROTARY = 2
    """A rotary axis that accepts angle units for position."""

    PROCESS = 3
    """A process on a process controller."""

    LAMP = 4
    """A lamp on a light source controller."""



# This file is generated. Do not modify by hand.
from enum import Enum


class ProcessControllerMode(Enum):
    """
    Mode of the process controller.
    """

    MANUAL = 0
    """Allows direct control of the output voltage."""

    PID = 1
    """Closed-loop PID control to maintain a setpoint."""

    PID_HEATER = 2
    """PID control optimized for heater loads."""

    ON_OFF = 3
    """Binary on/off switching based on a threshold."""



# This file is generated. Do not modify by hand.
from enum import Enum


class TriggerOperation(Enum):
    """
    Operation for trigger action.
    """

    SET_TO = 0
    """Assign the value to the setting (=)."""

    INCREMENT_BY = 1
    """Add the value to the current setting value (+=)."""

    DECREMENT_BY = 2
    """Subtract the value from the current setting value (-=)."""



# This file is generated. Do not modify by hand.
from enum import Enum


class DeviceDbSourceType(Enum):
    """
    Type of source of Device DB data.
    """

    WEB_SERVICE = 0
    """Fetch device information from a web service."""

    FILE = 1
    """Load device information from a local database file."""



# This file is generated. Do not modify by hand.
from enum import Enum


class IoPortType(Enum):
    """
    Kind of I/O pin to use.
    """

    NONE = 0
    """No port type specified."""

    ANALOG_INPUT = 1
    """An analog input channel that reads a voltage."""

    ANALOG_OUTPUT = 2
    """An analog output channel that outputs a voltage."""

    DIGITAL_INPUT = 3
    """A digital input channel that reads a boolean state."""

    DIGITAL_OUTPUT = 4
    """A digital output channel that controls a boolean state."""



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict, Optional
import decimal
import zaber_bson
from ..ascii.io_port_type import IoPortType


@dataclass
class SetIoPortLabelRequest:

    interface_id: int = 0

    device: int = 0

    port_type: IoPortType = next(first for first in IoPortType)

    channel_number: int = 0

    label: Optional[str] = None

    @staticmethod
    def zero_values() -> 'SetIoPortLabelRequest':
        return SetIoPortLabelRequest(
            interface_id=0,
            device=0,
            port_type=next(first for first in IoPortType),
            channel_number=0,
            label=None,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'SetIoPortLabelRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return SetIoPortLabelRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'interfaceId': int(self.interface_id),
            'device': int(self.device),
            'portType': self.port_type.value,
            'channelNumber': int(self.channel_number),
            'label': str(self.label) if self.label is not None else None,
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'SetIoPortLabelRequest':
        return SetIoPortLabelRequest(
            interface_id=data.get('interfaceId'),  # type: ignore
            device=data.get('device'),  # type: ignore
            port_type=IoPortType(data.get('portType')),  # type: ignore
            channel_number=data.get('channelNumber'),  # type: ignore
            label=data.get('label'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.interface_id is None:
            raise ValueError(f'Property "InterfaceId" of "SetIoPortLabelRequest" is None.')

        if not isinstance(self.interface_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "InterfaceId" of "SetIoPortLabelRequest" is not a number.')

        if int(self.interface_id) != self.interface_id:
            raise ValueError(f'Property "InterfaceId" of "SetIoPortLabelRequest" is not integer value.')

        if self.device is None:
            raise ValueError(f'Property "Device" of "SetIoPortLabelRequest" is None.')

        if not isinstance(self.device, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Device" of "SetIoPortLabelRequest" is not a number.')

        if int(self.device) != self.device:
            raise ValueError(f'Property "Device" of "SetIoPortLabelRequest" is not integer value.')

        if self.port_type is None:
            raise ValueError(f'Property "PortType" of "SetIoPortLabelRequest" is None.')

        if not isinstance(self.port_type, IoPortType):
            raise ValueError(f'Property "PortType" of "SetIoPortLabelRequest" is not an instance of "IoPortType".')

        if self.channel_number is None:
            raise ValueError(f'Property "ChannelNumber" of "SetIoPortLabelRequest" is None.')

        if not isinstance(self.channel_number, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "ChannelNumber" of "SetIoPortLabelRequest" is not a number.')

        if int(self.channel_number) != self.channel_number:
            raise ValueError(f'Property "ChannelNumber" of "SetIoPortLabelRequest" is not integer value.')

        if self.label is not None:
            if not isinstance(self.label, str):
                raise ValueError(f'Property "Label" of "SetIoPortLabelRequest" is not a string.')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict
import decimal
import zaber_bson
from .message_type import MessageType


@dataclass
class Response:
    """
    Response message from the device.
    """

    device_address: int
    """
    Number of the device that sent the message.
    """

    axis_number: int
    """
    Number of the axis which the response applies to. Zero denotes device scope.
    """

    reply_flag: str
    """
    The reply flag indicates if the request was accepted (OK) or rejected (RJ).
    Does not apply to info messages.
    """

    status: str
    """
    The device status contains BUSY when the axis is moving and IDLE otherwise.
    Does not apply to info messages.
    """

    warning_flag: str
    """
    The warning flag contains the highest priority warning currently active for the device or axis.
    Does not apply to info messages.
    """

    data: str
    """
    Response data which varies depending on the request.
    """

    message_type: MessageType
    """
    Type of the response received (only Reply or Info).
    """

    @staticmethod
    def zero_values() -> 'Response':
        return Response(
            device_address=0,
            axis_number=0,
            reply_flag="",
            status="",
            warning_flag="",
            data="",
            message_type=next(first for first in MessageType),
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'Response':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return Response.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'deviceAddress': int(self.device_address),
            'axisNumber': int(self.axis_number),
            'replyFlag': str(self.reply_flag or ''),
            'status': str(self.status or ''),
            'warningFlag': str(self.warning_flag or ''),
            'data': str(self.data or ''),
            'messageType': self.message_type.value,
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'Response':
        return Response(
            device_address=data.get('deviceAddress'),  # type: ignore
            axis_number=data.get('axisNumber'),  # type: ignore
            reply_flag=data.get('replyFlag'),  # type: ignore
            status=data.get('status'),  # type: ignore
            warning_flag=data.get('warningFlag'),  # type: ignore
            data=data.get('data'),  # type: ignore
            message_type=MessageType(data.get('messageType')),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.device_address is None:
            raise ValueError(f'Property "DeviceAddress" of "Response" is None.')

        if not isinstance(self.device_address, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "DeviceAddress" of "Response" is not a number.')

        if int(self.device_address) != self.device_address:
            raise ValueError(f'Property "DeviceAddress" of "Response" is not integer value.')

        if self.axis_number is None:
            raise ValueError(f'Property "AxisNumber" of "Response" is None.')

        if not isinstance(self.axis_number, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "AxisNumber" of "Response" is not a number.')

        if int(self.axis_number) != self.axis_number:
            raise ValueError(f'Property "AxisNumber" of "Response" is not integer value.')

        if self.reply_flag is not None:
            if not isinstance(self.reply_flag, str):
                raise ValueError(f'Property "ReplyFlag" of "Response" is not a string.')

        if self.status is not None:
            if not isinstance(self.status, str):
                raise ValueError(f'Property "Status" of "Response" is not a string.')

        if self.warning_flag is not None:
            if not isinstance(self.warning_flag, str):
                raise ValueError(f'Property "WarningFlag" of "Response" is not a string.')

        if self.data is not None:
            if not isinstance(self.data, str):
                raise ValueError(f'Property "Data" of "Response" is not a string.')

        if self.message_type is None:
            raise ValueError(f'Property "MessageType" of "Response" is None.')

        if not isinstance(self.message_type, MessageType):
            raise ValueError(f'Property "MessageType" of "Response" is not an instance of "MessageType".')



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
from collections.abc import Iterable
import zaber_bson
from .test_variant import TestVariant, TestVariantWireFormat


@dataclass
class TestDtoWithVariants:
    """
    A test DTO that includes variant types.
    This is used to verify the correct handling of variant types as members of other DTO types.
    """

    variant_array: List[TestVariant] = field(default_factory=list)
    """
    Array of variants.
    """

    the_variant: Optional[TestVariant] = None
    """
    Single variant instance.
    """

    @staticmethod
    def zero_values() -> 'TestDtoWithVariants':
        return TestDtoWithVariants(
            the_variant=None,
            variant_array=[],
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'TestDtoWithVariants':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return TestDtoWithVariants.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'theVariant': TestVariantWireFormat(self.the_variant).to_dict() if self.the_variant is not None else None,
            'variantArray': [TestVariantWireFormat(item).to_dict() for item in self.variant_array] if self.variant_array is not None else [],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'TestDtoWithVariants':
        return TestDtoWithVariants(
            the_variant=TestVariantWireFormat.from_dict(data.get('theVariant')).convert_back() if data.get('theVariant') is not None else None,  # type: ignore
            variant_array=[TestVariantWireFormat.from_dict(item).convert_back() for item in data.get('variantArray')],  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.the_variant is not None:
            TestVariantWireFormat(self.the_variant).validate()

        if self.variant_array is not None:
            if not isinstance(self.variant_array, Iterable):
                raise ValueError('Property "VariantArray" of "TestDtoWithVariants" is not iterable.')

            for i, variant_array_item in enumerate(self.variant_array):
                if variant_array_item is not None:
                    TestVariantWireFormat(variant_array_item).validate()



# This file is generated. Do not modify by hand.
from enum import Enum


class InterfaceType(Enum):

    SERIAL_PORT = 0

    TCP = 1

    CUSTOM = 2

    IOT = 3

    NETWORK_SHARE = 4

    MOCK = 5



# This file is generated. Do not modify by hand.
from enum import Enum


class MessageType(Enum):
    """
    Denotes type of the response message.
    For more information refer to:
    [ASCII Protocol Manual](https://www.zaber.com/protocol-manual#topic_message_format).
    """

    REPLY = 0
    """A direct response to a command sent to the device."""

    INFO = 1
    """An additional data message supplementing a reply."""

    ALERT = 2
    """An unsolicited message from the device reporting a status change."""



# This file is generated. Do not modify by hand.
from enum import Enum


class ServoTuningParamset(Enum):
    """
    Servo Tuning Parameter Set to target.
    """

    LIVE = 0
    """The currently active servo tuning parameters."""

    P_1 = 1
    """Stored parameter set 1."""

    P_2 = 2
    """Stored parameter set 2."""

    P_3 = 3
    """Stored parameter set 3."""

    P_4 = 4
    """Stored parameter set 4."""

    P_5 = 5
    """Stored parameter set 5."""

    P_6 = 6
    """Stored parameter set 6."""

    P_7 = 7
    """Stored parameter set 7."""

    P_8 = 8
    """Stored parameter set 8."""

    P_9 = 9
    """Stored parameter set 9."""

    STAGING = 10
    """A temporary working area for preparing tuning changes."""

    DEFAULT = 11
    """The factory default parameter set."""



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass, field
from typing import Any, Dict
import decimal
import zaber_bson
from ..ascii.unknown_response_event import UnknownResponseEvent


@dataclass
class UnknownResponseEventWrapper:

    interface_id: int = 0

    session_id: int = 0
    """
    The id of the connection session.
    """

    unknown_response: UnknownResponseEvent = field(default_factory=UnknownResponseEvent.zero_values)

    @staticmethod
    def zero_values() -> 'UnknownResponseEventWrapper':
        return UnknownResponseEventWrapper(
            interface_id=0,
            session_id=0,
            unknown_response=UnknownResponseEvent.zero_values(),
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'UnknownResponseEventWrapper':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return UnknownResponseEventWrapper.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'interfaceId': int(self.interface_id),
            'sessionId': int(self.session_id),
            'unknownResponse': self.unknown_response.to_dict(),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'UnknownResponseEventWrapper':
        return UnknownResponseEventWrapper(
            interface_id=data.get('interfaceId'),  # type: ignore
            session_id=data.get('sessionId'),  # type: ignore
            unknown_response=UnknownResponseEvent.from_dict(data.get('unknownResponse')),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.interface_id is None:
            raise ValueError(f'Property "InterfaceId" of "UnknownResponseEventWrapper" is None.')

        if not isinstance(self.interface_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "InterfaceId" of "UnknownResponseEventWrapper" is not a number.')

        if int(self.interface_id) != self.interface_id:
            raise ValueError(f'Property "InterfaceId" of "UnknownResponseEventWrapper" is not integer value.')

        if self.session_id is None:
            raise ValueError(f'Property "SessionId" of "UnknownResponseEventWrapper" is None.')

        if not isinstance(self.session_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "SessionId" of "UnknownResponseEventWrapper" is not a number.')

        if int(self.session_id) != self.session_id:
            raise ValueError(f'Property "SessionId" of "UnknownResponseEventWrapper" is not integer value.')

        if self.unknown_response is None:
            raise ValueError(f'Property "UnknownResponse" of "UnknownResponseEventWrapper" is None.')

        if not isinstance(self.unknown_response, UnknownResponseEvent):
            raise ValueError(f'Property "UnknownResponse" of "UnknownResponseEventWrapper" is not an instance of "UnknownResponseEvent".')

        self.unknown_response.validate()



# This file is generated. Do not modify by hand.
from enum import Enum


class BinarySettings(Enum):
    """
    Named constants for all Zaber Binary protocol settings.
    """

    ACCELERATION = 0

    ACCELERATION_ONLY = 1

    ACTIVE_AXIS = 2

    ALIAS_NUMBER = 3

    ANALOG_INPUT_COUNT = 4

    ANALOG_OUTPUT_COUNT = 5

    AUTO_HOME_DISABLED_MODE = 6

    AUTO_REPLY_DISABLED_MODE = 7

    AXIS_DEVICE_NUMBER = 8

    AXIS_INVERSION = 9

    AXIS_VELOCITY_PROFILE = 10

    AXIS_VELOCITY_SCALE = 11

    BAUD_RATE = 12

    CALIBRATED_ENCODER_COUNT = 13

    CALIBRATION_ERROR = 14

    CALIBRATION_TYPE = 15

    CLOSED_LOOP_MODE = 16

    CURRENT_POSITION = 17

    CYCLE_DISTANCE = 18

    DECELERATION_ONLY = 19

    DEVICE_DIRECTION = 20

    DEVICE_ID = 21

    DEVICE_MODE = 22

    DIGITAL_INPUT_COUNT = 23

    DIGITAL_OUTPUT_COUNT = 24

    ENCODER_COUNT = 25

    ENCODER_POSITION = 26

    FILTER_HOLDER_ID = 27

    FIRMWARE_BUILD = 28

    FIRMWARE_VERSION = 29

    HOLD_CURRENT = 30

    HOME_OFFSET = 31

    HOME_SENSOR_TYPE = 32

    HOME_SPEED = 33

    HOME_STATUS = 34

    INDEX_DISTANCE = 35

    JOYSTICK_CALIBRATION_MODE = 36

    KNOB_DIRECTION = 37

    KNOB_DISABLED_MODE = 38

    KNOB_JOG_SIZE = 39

    KNOB_MOVEMENT_MODE = 40

    KNOB_VELOCITY_PROFILE = 41

    KNOB_VELOCITY_SCALE = 42

    LOCK_STATE = 43

    MANUAL_MOVE_TRACKING_DISABLED_MODE = 44

    MAXIMUM_POSITION = 45

    MAXIMUM_RELATIVE_MOVE = 46

    MESSAGE_ID_MODE = 47

    MICROSTEP_RESOLUTION = 48

    MINIMUM_POSITION = 49

    MOVE_TRACKING_MODE = 50

    MOVE_TRACKING_PERIOD = 51

    PARK_STATE = 52

    PERIPHERAL_ID = 53

    PERIPHERAL_ID_PENDING = 54

    PERIPHERAL_SERIAL_NUMBER = 55

    PERIPHERAL_SERIAL_PENDING = 56

    POWER_SUPPLY_VOLTAGE = 57

    PROTOCOL = 58

    RUNNING_CURRENT = 59

    SERIAL_NUMBER = 60

    SLIP_TRACKING_PERIOD = 61

    STALL_TIMEOUT = 62

    STATUS = 63

    TARGET_SPEED = 64



# This file is generated. Do not modify by hand.
from enum import Enum


class DeviceType(Enum):
    """
    Denotes type of an device and units it accepts.
    """

    UNKNOWN = 0
    """Device type could not be determined."""

    LINEAR = 1
    """A linear device that accepts length units for position."""

    ROTARY = 2
    """A rotary device that accepts angle units for position."""



# This file is generated. Do not modify by hand.
from enum import Enum


class DevicePortType(Enum):
    """
    Type of TCP port used to connect to a device on the network.
    """

    DEVICE = 0
    """TCP port that communicates only with the connected device."""

    DEVICE_CHAIN = 1
    """TCP port that communicates with the connected device and all chained devices."""



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

# pylint: disable=too-many-arguments

from typing import Generator, List, Any, Optional, TypeVar
import asyncio
from reactivex.subject import ReplaySubject
from reactivex import operators as rxop, Observable

from ..call import call, call_async, call_sync
from ..convert_exception import convert_exception
from ..events import filter_events
from ..exceptions.motion_lib_exception import MotionLibException

from .device import Device
from ..dto.binary.command_code import CommandCode
from ..dto.binary.message import Message
from ..dto.binary.reply_only_event import ReplyOnlyEvent
from ..dto.binary.unknown_response_event import UnknownResponseEvent
from ..dto import requests as dto


TConnectionEvents = TypeVar(
    "TConnectionEvents",
    dto.UnknownBinaryResponseEventWrapper,
    dto.BinaryReplyOnlyEventWrapper,
    dto.DisconnectedEvent)


class Connection:
    """
    Class representing access to particular connection (serial port, TCP connection) using the legacy Binary protocol.
    """

    @property
    def unknown_response(self) -> Observable[UnknownResponseEvent]:
        """
        Event invoked when a response from a device cannot be matched to any known request.
        """
        return self._unknown_response

    @property
    def reply_only(self) -> Observable[ReplyOnlyEvent]:
        """
        Event invoked when a reply-only command such as a move tracking message is received from a device.
        """
        return self._reply_only

    @property
    def disconnected(self) -> Observable[MotionLibException]:
        """
        Event invoked when connection is interrupted or closed.
        """
        return self._disconnected

    DEFAULT_BAUD_RATE = 9600
    """
    Default baud rate for serial connections.
    """

    TCP_PORT_CHAIN = 55550
    """
    Commands sent over this port are forwarded to the device chain.
    The bandwidth may be limited as the commands are forwarded over a serial connection.
    """

    TCP_PORT_DEVICE_ONLY = 55551
    """
    Commands send over this port are processed only by the device
    and not forwarded to the rest of the chain.
    Using this port typically makes the communication faster.
    """

    @property
    def interface_id(self) -> int:
        """
        The interface ID identifies thisConnection instance with the underlying library.
        """
        return self._interface_id

    @property
    def is_open(self) -> bool:
        """
        Returns whether the connection is open.
        Does not guarantee that the subsequent requests will succeed.
        """
        return self.__retrieve_is_open()

    def __init__(self, interface_id: int):
        self._interface_id: int = interface_id
        self.__setup_events()

    @staticmethod
    def open_serial_port(
            port_name: str,
            baud_rate: int = DEFAULT_BAUD_RATE,
            use_message_ids: bool = False
    ) -> 'Connection':
        """
        Opens a serial port.

        Args:
            port_name: Name of the port to open.
            baud_rate: Optional baud rate (defaults to 9600).
            use_message_ids: Enable use of message IDs (defaults to disabled).
                All your devices must be pre-configured to match.

        Returns:
            An object representing the port.
        """
        request = dto.OpenBinaryInterfaceRequest(
            interface_type=dto.InterfaceType.SERIAL_PORT,
            port_name=port_name,
            baud_rate=baud_rate,
            use_message_ids=use_message_ids,
        )
        response = call(
            "binary/interface/open",
            request,
            dto.OpenInterfaceResponse.from_binary)
        return Connection(response.interface_id)

    @staticmethod
    def open_serial_port_async(
            port_name: str,
            baud_rate: int = DEFAULT_BAUD_RATE,
            use_message_ids: bool = False
    ) -> 'AsyncBinaryConnectionOpener':
        """
        Opens a serial port.

        Args:
            port_name: Name of the port to open.
            baud_rate: Optional baud rate (defaults to 9600).
            use_message_ids: Enable use of message IDs (defaults to disabled).
                All your devices must be pre-configured to match.

        Returns:
            An object representing the port.
        """
        request = dto.OpenBinaryInterfaceRequest(
            interface_type=dto.InterfaceType.SERIAL_PORT,
            port_name=port_name,
            baud_rate=baud_rate,
            use_message_ids=use_message_ids,
        )
        return AsyncBinaryConnectionOpener(request)

    @staticmethod
    def open_tcp(
            host_name: str,
            port: int = TCP_PORT_CHAIN,
            use_message_ids: bool = False
    ) -> 'Connection':
        """
        Opens a TCP connection.

        Args:
            host_name: Hostname or IP address.
            port: Optional port number (defaults to 55550).
            use_message_ids: Enable use of message IDs (defaults to disabled).
                All your devices must be pre-configured to match.

        Returns:
            An object representing the connection.
        """
        request = dto.OpenBinaryInterfaceRequest(
            interface_type=dto.InterfaceType.TCP,
            host_name=host_name,
            port=port,
            use_message_ids=use_message_ids,
        )
        response = call(
            "binary/interface/open",
            request,
            dto.OpenInterfaceResponse.from_binary)
        return Connection(response.interface_id)

    @staticmethod
    def open_tcp_async(
            host_name: str,
            port: int = TCP_PORT_CHAIN,
            use_message_ids: bool = False
    ) -> 'AsyncBinaryConnectionOpener':
        """
        Opens a TCP connection.

        Args:
            host_name: Hostname or IP address.
            port: Optional port number (defaults to 55550).
            use_message_ids: Enable use of message IDs (defaults to disabled).
                All your devices must be pre-configured to match.

        Returns:
            An object representing the connection.
        """
        request = dto.OpenBinaryInterfaceRequest(
            interface_type=dto.InterfaceType.TCP,
            host_name=host_name,
            port=port,
            use_message_ids=use_message_ids,
        )
        return AsyncBinaryConnectionOpener(request)

    def close(
            self
    ) -> None:
        """
        Close the connection.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=self.interface_id,
        )
        call("interface/close", request)

    async def close_async(
            self
    ) -> None:
        """
        Close the connection.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=self.interface_id,
        )
        await call_async("interface/close", request)

    def generic_command(
            self,
            device: int,
            command: CommandCode,
            data: int = 0,
            timeout: float = 0.0,
            check_errors: bool = True
    ) -> Message:
        """
        Sends a generic Binary command to this connection.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            device: Device address to send the command to. Use zero for broadcast.
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
            timeout: Number of seconds to wait for a response from the device. 0 or negative defaults to 0.5s.
            check_errors: Controls whether to throw an exception when the device rejects the command.

        Returns:
            A response to the command.
        """
        request = dto.GenericBinaryRequest(
            interface_id=self.interface_id,
            device=device,
            command=command,
            data=data,
            timeout=timeout,
            check_errors=check_errors,
        )
        response = call(
            "binary/interface/generic_command",
            request,
            Message.from_binary)
        return response

    async def generic_command_async(
            self,
            device: int,
            command: CommandCode,
            data: int = 0,
            timeout: float = 0.0,
            check_errors: bool = True
    ) -> Message:
        """
        Sends a generic Binary command to this connection.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            device: Device address to send the command to. Use zero for broadcast.
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
            timeout: Number of seconds to wait for a response from the device. 0 or negative defaults to 0.5s.
            check_errors: Controls whether to throw an exception when the device rejects the command.

        Returns:
            A response to the command.
        """
        request = dto.GenericBinaryRequest(
            interface_id=self.interface_id,
            device=device,
            command=command,
            data=data,
            timeout=timeout,
            check_errors=check_errors,
        )
        response = await call_async(
            "binary/interface/generic_command",
            request,
            Message.from_binary)
        return response

    def generic_command_no_response(
            self,
            device: int,
            command: CommandCode,
            data: int = 0
    ) -> None:
        """
        Sends a generic Binary command to this connection without expecting a response.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            device: Device address to send the command to. Use zero for broadcast.
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
        """
        request = dto.GenericBinaryRequest(
            interface_id=self.interface_id,
            device=device,
            command=command,
            data=data,
        )
        call("binary/interface/generic_command_no_response", request)

    async def generic_command_no_response_async(
            self,
            device: int,
            command: CommandCode,
            data: int = 0
    ) -> None:
        """
        Sends a generic Binary command to this connection without expecting a response.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            device: Device address to send the command to. Use zero for broadcast.
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
        """
        request = dto.GenericBinaryRequest(
            interface_id=self.interface_id,
            device=device,
            command=command,
            data=data,
        )
        await call_async("binary/interface/generic_command_no_response", request)

    def generic_command_multi_response(
            self,
            command: CommandCode,
            data: int = 0,
            timeout: float = 0.0,
            check_errors: bool = True
    ) -> List[Message]:
        """
        Sends a generic Binary command to this connection and expects responses from one or more devices.
        Responses are returned in order of arrival.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
            timeout: Number of seconds to wait for all responses from the device chain. 0 or negative defaults to 0.5s.
            check_errors: Controls whether to throw an exception when any device rejects the command.

        Returns:
            All responses to the command.
        """
        request = dto.GenericBinaryRequest(
            interface_id=self.interface_id,
            command=command,
            data=data,
            timeout=timeout,
            check_errors=check_errors,
        )
        response = call(
            "binary/interface/generic_command_multi_response",
            request,
            dto.BinaryMessageCollection.from_binary)
        return response.messages

    async def generic_command_multi_response_async(
            self,
            command: CommandCode,
            data: int = 0,
            timeout: float = 0.0,
            check_errors: bool = True
    ) -> List[Message]:
        """
        Sends a generic Binary command to this connection and expects responses from one or more devices.
        Responses are returned in order of arrival.
        For more information please refer to the
        [Binary Protocol Manual](https://www.zaber.com/protocol-manual?protocol=Binary#topic_quick_command_reference).

        Args:
            command: Command to send.
            data: Optional data argument to the command. Defaults to zero.
            timeout: Number of seconds to wait for all responses from the device chain. 0 or negative defaults to 0.5s.
            check_errors: Controls whether to throw an exception when any device rejects the command.

        Returns:
            All responses to the command.
        """
        request = dto.GenericBinaryRequest(
            interface_id=self.interface_id,
            command=command,
            data=data,
            timeout=timeout,
            check_errors=check_errors,
        )
        response = await call_async(
            "binary/interface/generic_command_multi_response",
            request,
            dto.BinaryMessageCollection.from_binary)
        return response.messages

    def renumber_devices(
            self
    ) -> int:
        """
        Renumbers devices present on this connection. After renumbering, you must identify devices again.

        Returns:
            Total number of devices that responded to the renumber.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=self.interface_id,
        )
        response = call(
            "binary/device/renumber",
            request,
            dto.IntResponse.from_binary)
        return response.value

    async def renumber_devices_async(
            self
    ) -> int:
        """
        Renumbers devices present on this connection. After renumbering, you must identify devices again.

        Returns:
            Total number of devices that responded to the renumber.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=self.interface_id,
        )
        response = await call_async(
            "binary/device/renumber",
            request,
            dto.IntResponse.from_binary)
        return response.value

    def detect_devices(
            self,
            identify_devices: bool = True
    ) -> List[Device]:
        """
        Attempts to detect any devices present on this connection.

        Args:
            identify_devices: Determines whether device identification should be performed as well.

        Returns:
            Array of detected devices.
        """
        request = dto.BinaryDeviceDetectRequest(
            interface_id=self.interface_id,
            identify_devices=identify_devices,
        )
        response = call(
            "binary/device/detect",
            request,
            dto.BinaryDeviceDetectResponse.from_binary)
        return list(map(self.get_device, response.devices))

    async def detect_devices_async(
            self,
            identify_devices: bool = True
    ) -> List[Device]:
        """
        Attempts to detect any devices present on this connection.

        Args:
            identify_devices: Determines whether device identification should be performed as well.

        Returns:
            Array of detected devices.
        """
        request = dto.BinaryDeviceDetectRequest(
            interface_id=self.interface_id,
            identify_devices=identify_devices,
        )
        response = await call_async(
            "binary/device/detect",
            request,
            dto.BinaryDeviceDetectResponse.from_binary)
        return list(map(self.get_device, response.devices))

    def get_device(
            self,
            device_address: int
    ) -> Device:
        """
        Gets a Device class instance which allows you to control a particular device on this connection.
        Devices are numbered from 1.

        Args:
            device_address: Address of device intended to control. Address is configured for each device.

        Returns:
            Device instance.
        """
        if device_address <= 0:
            raise ValueError('Invalid value; physical devices are numbered from 1.')

        return Device(self, device_address)

    def __retrieve_is_open(
            self
    ) -> bool:
        """
        Returns is open.

        Returns:
            Is open.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=self.interface_id,
        )
        response = call_sync(
            "interface/get_is_open",
            request,
            dto.BoolResponse.from_binary)
        return response.value

    def __repr__(
            self
    ) -> str:
        """
        Returns a string that represents the connection.

        Returns:
            A string that represents the connection.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=self.interface_id,
        )
        response = call_sync(
            "interface/to_string",
            request,
            dto.StringResponse.from_binary)
        return response.value

    @staticmethod
    def __free(
            interface_id: int
    ) -> None:
        """
        Releases native resources of the connection.

        Args:
            interface_id: The ID of the connection.
        """
        request = dto.InterfaceEmptyRequest(
            interface_id=interface_id,
        )
        call_sync("interface/free", request)

    def __enter__(self) -> 'Connection':
        """ __enter__ """
        return self

    def __exit__(self, _type: Any, _value: Any, _traceback: Any) -> None:
        """ __exit__ """
        self.close()

    def __setup_events(self) -> None:
        def filter_connection_event(
            data: TConnectionEvents,
        ) -> bool:
            return data.interface_id == self._interface_id

        self._disconnected = ReplaySubject[MotionLibException]()  # terminates all the events

        unknown_response = filter_events('binary/interface/unknown_response', dto.UnknownBinaryResponseEventWrapper)
        self._unknown_response = unknown_response.pipe(
            rxop.filter(filter_connection_event),
            rxop.take_until(self.disconnected),
            rxop.map(map_unknown_response)
        )

        reply_only = filter_events('binary/interface/reply_only', dto.BinaryReplyOnlyEventWrapper)
        self._reply_only = reply_only.pipe(
            rxop.filter(filter_connection_event),
            rxop.take_until(self.disconnected),
            rxop.map(map_reply_only)
        )

        disconnected = filter_events('interface/disconnected', dto.DisconnectedEvent)
        disconnected.pipe(
            rxop.filter(filter_connection_event),
            rxop.take(1),
            rxop.map(map_disconnect)
        ).subscribe(self._disconnected)

    def __del__(self) -> None:
        Connection.__free(self._interface_id)


class AsyncBinaryConnectionOpener:
    '''Provides a connection in an asynchronous context using `await` or `async with`'''
    def __init__(self, request: dto.OpenBinaryInterfaceRequest) -> None:
        self._request = request
        self._resource: Optional[Connection] = None

    async def _create_resource(self) -> Connection:
        task = asyncio.ensure_future(call_async(
            "binary/interface/open",
            self._request,
            dto.OpenInterfaceResponse.from_binary))

        try:
            response = await asyncio.shield(task)
        except asyncio.CancelledError:
            async def cancel() -> None:
                try:
                    response = await task
                    await Connection(response.interface_id).close_async()
                except MotionLibException:
                    pass

            asyncio.ensure_future(cancel())
            raise

        return Connection(response.interface_id)

    def __await__(self) -> Generator[Any, None, 'Connection']:
        return self._create_resource().__await__()

    async def __aenter__(self) -> 'Connection':
        self._resource = await self._create_resource()
        return self._resource

    async def __aexit__(self, exc_type: Any, exc: Any, trace: Any) -> None:
        if self._resource is not None:
            await self._resource.close_async()


def map_unknown_response(event: dto.UnknownBinaryResponseEventWrapper) -> UnknownResponseEvent:
    return event.unknown_response


def map_reply_only(event: dto.BinaryReplyOnlyEventWrapper) -> ReplyOnlyEvent:
    return event.reply


def map_disconnect(event: dto.DisconnectedEvent) -> MotionLibException:
    return convert_exception(event.error_type, event.error_message)



# This file is generated. Do not modify by hand.
from enum import Enum


class Errors(Enum):

    BAD_COMMAND = 0

    BAD_DATA = 1

    BINARY_COMMAND_FAILED = 2

    COMMAND_FAILED = 3

    COMMAND_PREEMPTED = 4

    COMMAND_TOO_LONG = 5

    CONNECTION_CLOSED = 6

    CONNECTION_FAILED = 7

    CONVERSION_FAILED = 8

    DEVICE_ADDRESS_CONFLICT = 9

    DEVICE_BUSY = 10

    DEVICE_DB_FAILED = 11

    DEVICE_DETECTION_FAILED = 12

    DEVICE_FAILED = 13

    DEVICE_NOT_IDENTIFIED = 14

    DRIVER_DISABLED = 15

    G_CODE_EXECUTION = 16

    G_CODE_SYNTAX = 17

    INCOMPATIBLE_SHARED_LIBRARY = 18

    INTERNAL_ERROR = 19

    INVALID_ARGUMENT = 20

    INVALID_CSV_DATA = 21

    INVALID_DATA = 22

    INVALID_OPERATION = 23

    INVALID_PACKET = 24

    INVALID_PARK_STATE = 25

    INVALID_REQUEST_DATA = 26

    INVALID_RESPONSE = 27

    IO_CHANNEL_OUT_OF_RANGE = 28

    IO_FAILED = 29

    LOCKSTEP_ENABLED = 30

    LOCKSTEP_NOT_ENABLED = 31

    MOVEMENT_FAILED = 32

    MOVEMENT_INTERRUPTED = 33

    NO_DEVICE_FOUND = 34

    NO_VALUE_FOR_KEY = 35

    NOT_SUPPORTED = 36

    OPERATION_FAILED = 37

    OS_FAILED = 38

    OUT_OF_REQUEST_IDS = 39

    PVT_DISCONTINUITY = 40

    PVT_EXECUTION = 41

    PVT_MODE = 42

    PVT_MOVEMENT_FAILED = 43

    PVT_MOVEMENT_INTERRUPTED = 44

    PVT_SEQUENCE_GENERATION_FAILED = 45

    PVT_SETUP_FAILED = 46

    REMOTE_MODE = 47

    REQUEST_TIMEOUT = 48

    SERIAL_PORT_BUSY = 49

    SET_DEVICE_STATE_FAILED = 50

    SET_PERIPHERAL_STATE_FAILED = 51

    SETTING_NOT_FOUND = 52

    STREAM_DISCONTINUITY = 53

    STREAM_EXECUTION = 54

    STREAM_MODE = 55

    STREAM_MOVEMENT_FAILED = 56

    STREAM_MOVEMENT_INTERRUPTED = 57

    STREAM_SETUP_FAILED = 58

    TIMEOUT = 59

    TRANSPORT_ALREADY_USED = 60

    UNKNOWN_REQUEST = 61



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional
from collections.abc import Iterable
import zaber_bson
from ..ascii.measurement_sequence import MeasurementSequence
from ..ascii.optional_measurement_sequence import OptionalMeasurementSequence


@dataclass
class PvtGenerateVelocitiesRequest:

    positions: List[MeasurementSequence] = field(default_factory=list)

    times: MeasurementSequence = field(default_factory=MeasurementSequence.zero_values)

    times_relative: bool = False

    velocities: Optional[List[OptionalMeasurementSequence]] = None

    @staticmethod
    def zero_values() -> 'PvtGenerateVelocitiesRequest':
        return PvtGenerateVelocitiesRequest(
            positions=[],
            velocities=None,
            times=MeasurementSequence.zero_values(),
            times_relative=False,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'PvtGenerateVelocitiesRequest':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return PvtGenerateVelocitiesRequest.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'positions': [item.to_dict() for item in self.positions] if self.positions is not None else [],
            'velocities': [item.to_dict() for item in self.velocities] if self.velocities is not None else [],
            'times': self.times.to_dict(),
            'timesRelative': bool(self.times_relative),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'PvtGenerateVelocitiesRequest':
        return PvtGenerateVelocitiesRequest(
            positions=[MeasurementSequence.from_dict(item) for item in data.get('positions')],  # type: ignore
            velocities=[OptionalMeasurementSequence.from_dict(item) for item in data.get('velocities')] if data.get('velocities') is not None else None,  # type: ignore
            times=MeasurementSequence.from_dict(data.get('times')),  # type: ignore
            times_relative=data.get('timesRelative'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.positions is not None:
            if not isinstance(self.positions, Iterable):
                raise ValueError('Property "Positions" of "PvtGenerateVelocitiesRequest" is not iterable.')

            for i, positions_item in enumerate(self.positions):
                if positions_item is None:
                    raise ValueError(f'Item {i} in property "Positions" of "PvtGenerateVelocitiesRequest" is None.')

                if not isinstance(positions_item, MeasurementSequence):
                    raise ValueError(f'Item {i} in property "Positions" of "PvtGenerateVelocitiesRequest" is not an instance of "MeasurementSequence".')

                positions_item.validate()

        if self.velocities is not None:
            if not isinstance(self.velocities, Iterable):
                raise ValueError('Property "Velocities" of "PvtGenerateVelocitiesRequest" is not iterable.')

            for i, velocities_item in enumerate(self.velocities):
                if velocities_item is None:
                    raise ValueError(f'Item {i} in property "Velocities" of "PvtGenerateVelocitiesRequest" is None.')

                if not isinstance(velocities_item, OptionalMeasurementSequence):
                    raise ValueError(f'Item {i} in property "Velocities" of "PvtGenerateVelocitiesRequest" is not an instance of "OptionalMeasurementSequence".')

                velocities_item.validate()

        if self.times is None:
            raise ValueError(f'Property "Times" of "PvtGenerateVelocitiesRequest" is None.')

        if not isinstance(self.times, MeasurementSequence):
            raise ValueError(f'Property "Times" of "PvtGenerateVelocitiesRequest" is not an instance of "MeasurementSequence".')

        self.times.validate()



# This file is generated. Do not modify by hand.
from enum import Enum


class StreamMode(Enum):
    """
    Mode of a stream.
    """

    DISABLED = 0
    """The stream is not set up."""

    STORE = 1
    """Actions are queued into a stream buffer for later playback."""

    STORE_ARBITRARY_AXES = 2
    """Actions are queued into a stream buffer for later playback (axes not specified)."""

    LIVE = 3
    """Actions are queued and executed on the device immediately."""



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict, List, Optional
import decimal
from collections.abc import Iterable
import zaber_bson
from ..firmware_version import FirmwareVersion
from .mock_peripheral import MockPeripheral


@dataclass
class MockDevice:
    """
    Definition of a mock device.
    """

    device_id: int
    """
    A valid Zaber device ID.
    """

    firmware_version: Optional[FirmwareVersion] = None
    """
    Version of the firmware. Defaults to device database latest. To omit minor or build version, set them to -1.
    """

    is_modified: Optional[bool] = None
    """
    The device has hardware modifications. Defaults to false.
    """

    resolution: Optional[int] = None
    """
    The number of microsteps per full step for integrated devices. Defaults to device database default.
    """

    peripherals: Optional[List[MockPeripheral]] = None
    """
    List of mock peripherals connected to the device.
    """

    @staticmethod
    def zero_values() -> 'MockDevice':
        return MockDevice(
            device_id=0,
            firmware_version=None,
            is_modified=None,
            resolution=None,
            peripherals=None,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'MockDevice':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return MockDevice.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'deviceId': int(self.device_id),
            'firmwareVersion': self.firmware_version.to_dict() if self.firmware_version is not None else None,
            'isModified': bool(self.is_modified) if self.is_modified is not None else None,
            'resolution': int(self.resolution) if self.resolution is not None else None,
            'peripherals': [item.to_dict() for item in self.peripherals] if self.peripherals is not None else [],
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'MockDevice':
        return MockDevice(
            device_id=data.get('deviceId'),  # type: ignore
            firmware_version=FirmwareVersion.from_dict(data.get('firmwareVersion')) if data.get('firmwareVersion') is not None else None,  # type: ignore
            is_modified=data.get('isModified'),  # type: ignore
            resolution=data.get('resolution'),  # type: ignore
            peripherals=[MockPeripheral.from_dict(item) for item in data.get('peripherals')] if data.get('peripherals') is not None else None,  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.device_id is None:
            raise ValueError(f'Property "DeviceId" of "MockDevice" is None.')

        if not isinstance(self.device_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "DeviceId" of "MockDevice" is not a number.')

        if int(self.device_id) != self.device_id:
            raise ValueError(f'Property "DeviceId" of "MockDevice" is not integer value.')

        if self.firmware_version is not None:
            if not isinstance(self.firmware_version, FirmwareVersion):
                raise ValueError(f'Property "FirmwareVersion" of "MockDevice" is not an instance of "FirmwareVersion".')

            self.firmware_version.validate()

        if self.resolution is not None:
            if not isinstance(self.resolution, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "Resolution" of "MockDevice" is not a number.')

            if int(self.resolution) != self.resolution:
                raise ValueError(f'Property "Resolution" of "MockDevice" is not integer value.')

        if self.peripherals is not None:
            if not isinstance(self.peripherals, Iterable):
                raise ValueError('Property "Peripherals" of "MockDevice" is not iterable.')

            for i, peripherals_item in enumerate(self.peripherals):
                if peripherals_item is None:
                    raise ValueError(f'Item {i} in property "Peripherals" of "MockDevice" is None.')

                if not isinstance(peripherals_item, MockPeripheral):
                    raise ValueError(f'Item {i} in property "Peripherals" of "MockDevice" is not an instance of "MockPeripheral".')

                peripherals_item.validate()



# This file is generated. Do not modify by hand.
from enum import Enum


class PvtMode(Enum):
    """
    Mode of a PVT sequence.
    """

    DISABLED = 0
    """The PVT sequence is not set up."""

    STORE = 1
    """PVT points are queued into a buffer for later playback."""

    LIVE = 2
    """PVT points are queued and executed on the device immediately."""



# This file is generated. Do not modify by hand.
from enum import Enum


class StreamAxisType(Enum):
    """
    Denotes type of the stream axis.
    """

    PHYSICAL = 0
    """A regular physical axis of the device."""

    LOCKSTEP = 1
    """A lockstep group combining multiple physical axes."""



# This file is generated. Do not modify by hand.
from enum import Enum


class ProcessControllerSourceSensor(Enum):
    """
    The type of input sensor that provides feedback to the process controller.
    """

    THERMISTOR = 10
    """Use a thermistor sensor port on the device to measure temperature."""

    ANALOG_INPUT = 20
    """Use an analog input I/O channel on the device to measure voltage."""



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict, Optional
import decimal
import zaber_bson


@dataclass
class MockPeripheral:
    """
    Definition of a mock peripheral.
    """

    peripheral_id: int
    """
    A valid Zaber peripheral ID.
    """

    is_modified: Optional[bool] = None
    """
    The peripheral has hardware modifications. Defaults to false.
    """

    resolution: Optional[int] = None
    """
    The number of microsteps per full step for peripheral. Defaults to device database default.
    """

    @staticmethod
    def zero_values() -> 'MockPeripheral':
        return MockPeripheral(
            peripheral_id=0,
            is_modified=None,
            resolution=None,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'MockPeripheral':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return MockPeripheral.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'peripheralId': int(self.peripheral_id),
            'isModified': bool(self.is_modified) if self.is_modified is not None else None,
            'resolution': int(self.resolution) if self.resolution is not None else None,
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'MockPeripheral':
        return MockPeripheral(
            peripheral_id=data.get('peripheralId'),  # type: ignore
            is_modified=data.get('isModified'),  # type: ignore
            resolution=data.get('resolution'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.peripheral_id is None:
            raise ValueError(f'Property "PeripheralId" of "MockPeripheral" is None.')

        if not isinstance(self.peripheral_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "PeripheralId" of "MockPeripheral" is not a number.')

        if int(self.peripheral_id) != self.peripheral_id:
            raise ValueError(f'Property "PeripheralId" of "MockPeripheral" is not integer value.')

        if self.resolution is not None:
            if not isinstance(self.resolution, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "Resolution" of "MockPeripheral" is not a number.')

            if int(self.resolution) != self.resolution:
                raise ValueError(f'Property "Resolution" of "MockPeripheral" is not integer value.')



# This file is generated. Do not modify by hand.
from enum import Enum


class ErrorCode(Enum):
    """
    Named constants for all Zaber Binary protocol error codes.
    """

    CANNOT_HOME = 1

    DEVICE_NUMBER_INVALID = 2

    ADDRESS_INVALID = 5

    VOLTAGE_LOW = 14

    VOLTAGE_HIGH = 15

    STORED_POSITION_INVALID = 18

    ABSOLUTE_POSITION_INVALID = 20

    RELATIVE_POSITION_INVALID = 21

    VELOCITY_INVALID = 22

    AXIS_INVALID = 25

    AXIS_DEVICE_NUMBER_INVALID = 26

    INVERSION_INVALID = 27

    VELOCITY_PROFILE_INVALID = 28

    VELOCITY_SCALE_INVALID = 29

    LOAD_EVENT_INVALID = 30

    RETURN_EVENT_INVALID = 31

    JOYSTICK_CALIBRATION_MODE_INVALID = 33

    PERIPHERAL_ID_INVALID = 36

    RESOLUTION_INVALID = 37

    RUN_CURRENT_INVALID = 38

    HOLD_CURRENT_INVALID = 39

    MODE_INVALID = 40

    HOME_SPEED_INVALID = 41

    SPEED_INVALID = 42

    ACCELERATION_INVALID = 43

    MAXIMUM_POSITION_INVALID = 44

    CURRENT_POSITION_INVALID = 45

    MAXIMUM_RELATIVE_MOVE_INVALID = 46

    OFFSET_INVALID = 47

    ALIAS_INVALID = 48

    LOCK_STATE_INVALID = 49

    DEVICE_ID_UNKNOWN = 50

    SETTING_INVALID = 53

    COMMAND_INVALID = 64

    PARK_STATE_INVALID = 65

    TEMPERATURE_HIGH = 67

    DIGITAL_INPUT_PIN_INVALID = 68

    DIGITAL_OUTPUT_PIN_INVALID = 71

    DIGITAL_OUTPUT_MASK_INVALID = 74

    ANALOG_INPUT_PIN_INVALID = 76

    MOVE_INDEX_NUMBER_INVALID = 78

    INDEX_DISTANCE_INVALID = 79

    CYCLE_DISTANCE_INVALID = 80

    FILTER_HOLDER_ID_INVALID = 81

    ABSOLUTE_FORCE_INVALID = 87

    AUTO_REPLY_DISABLED_MODE_INVALID = 101

    MESSAGE_ID_MODE_INVALID = 102

    HOME_STATUS_INVALID = 103

    HOME_SENSOR_TYPE_INVALID = 104

    AUTO_HOME_DISABLED_MODE_INVALID = 105

    MINIMUM_POSITION_INVALID = 106

    KNOB_DISABLED_MODE_INVALID = 107

    KNOB_DIRECTION_INVALID = 108

    KNOB_MOVEMENT_MODE_INVALID = 109

    KNOB_JOG_SIZE_INVALID = 110

    KNOB_VELOCITY_SCALE_INVALID = 111

    KNOB_VELOCITY_PROFILE_INVALID = 112

    ACCELERATION_ONLY_INVALID = 113

    DECELERATION_ONLY_INVALID = 114

    MOVE_TRACKING_MODE_INVALID = 115

    MANUAL_MOVE_TRACKING_DISABLED_MODE_INVALID = 116

    MOVE_TRACKING_PERIOD_INVALID = 117

    CLOSED_LOOP_MODE_INVALID = 118

    SLIP_TRACKING_PERIOD_INVALID = 119

    STALL_TIMEOUT_INVALID = 120

    DEVICE_DIRECTION_INVALID = 121

    BAUD_RATE_INVALID = 122

    PROTOCOL_INVALID = 123

    BAUD_RATE_OR_PROTOCOL_INVALID = 124

    BUSY = 255

    SYSTEM_ERROR = 257

    STORAGE_FULL = 401

    REGISTER_ADDRESS_INVALID = 701

    REGISTER_VALUE_INVALID = 702

    SAVE_POSITION_INVALID = 1600

    SAVE_POSITION_NOT_HOMED = 1601

    RETURN_POSITION_INVALID = 1700

    MOVE_POSITION_INVALID = 1800

    MOVE_POSITION_NOT_HOMED = 1801

    RELATIVE_POSITION_LIMITED = 2146

    SETTINGS_LOCKED = 3600

    BIT_1_INVALID = 4001

    BIT_2_INVALID = 4002

    DISABLE_AUTO_HOME_INVALID = 4008

    BIT_10_INVALID = 4010

    BIT_11_INVALID = 4011

    HOME_SWITCH_INVALID = 4012

    BIT_13_INVALID = 4013

    BIT_14_INVALID = 4014

    BIT_15_INVALID = 4015

    DEVICE_PARKED = 6501

    DRIVER_DISABLED = 9001



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict
import decimal
import zaber_bson
from .device_port_type import DevicePortType


@dataclass
class DeviceDiscoveryResult:
    """
    Represents result of mDNS discovery for devices connected to network.
    """

    hostname: str
    """
    Host name of the discoverd device.
    """

    address: str
    """
    IP address of the discoverd device.
    """

    port: int
    """
    Port number of the discovered device.
    """

    port_type: DevicePortType
    """
    Indicates if discovery result represents a single device or a device chain.
    """

    device_id: int
    """
    Device id of discovered device or head of discovered device chain.
    """

    serial_number: int
    """
    Serial number of discovered device or head of discovered device chain.
    """

    @staticmethod
    def zero_values() -> 'DeviceDiscoveryResult':
        return DeviceDiscoveryResult(
            hostname="",
            address="",
            port=0,
            port_type=next(first for first in DevicePortType),
            device_id=0,
            serial_number=0,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'DeviceDiscoveryResult':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return DeviceDiscoveryResult.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'hostname': str(self.hostname or ''),
            'address': str(self.address or ''),
            'port': int(self.port),
            'portType': self.port_type.value,
            'deviceId': int(self.device_id),
            'serialNumber': int(self.serial_number),
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'DeviceDiscoveryResult':
        return DeviceDiscoveryResult(
            hostname=data.get('hostname'),  # type: ignore
            address=data.get('address'),  # type: ignore
            port=data.get('port'),  # type: ignore
            port_type=DevicePortType(data.get('portType')),  # type: ignore
            device_id=data.get('deviceId'),  # type: ignore
            serial_number=data.get('serialNumber'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.hostname is not None:
            if not isinstance(self.hostname, str):
                raise ValueError(f'Property "Hostname" of "DeviceDiscoveryResult" is not a string.')

        if self.address is not None:
            if not isinstance(self.address, str):
                raise ValueError(f'Property "Address" of "DeviceDiscoveryResult" is not a string.')

        if self.port is None:
            raise ValueError(f'Property "Port" of "DeviceDiscoveryResult" is None.')

        if not isinstance(self.port, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Port" of "DeviceDiscoveryResult" is not a number.')

        if int(self.port) != self.port:
            raise ValueError(f'Property "Port" of "DeviceDiscoveryResult" is not integer value.')

        if self.port_type is None:
            raise ValueError(f'Property "PortType" of "DeviceDiscoveryResult" is None.')

        if not isinstance(self.port_type, DevicePortType):
            raise ValueError(f'Property "PortType" of "DeviceDiscoveryResult" is not an instance of "DevicePortType".')

        if self.device_id is None:
            raise ValueError(f'Property "DeviceId" of "DeviceDiscoveryResult" is None.')

        if not isinstance(self.device_id, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "DeviceId" of "DeviceDiscoveryResult" is not a number.')

        if int(self.device_id) != self.device_id:
            raise ValueError(f'Property "DeviceId" of "DeviceDiscoveryResult" is not integer value.')

        if self.serial_number is None:
            raise ValueError(f'Property "SerialNumber" of "DeviceDiscoveryResult" is None.')

        if not isinstance(self.serial_number, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "SerialNumber" of "DeviceDiscoveryResult" is not a number.')

        if int(self.serial_number) != self.serial_number:
            raise ValueError(f'Property "SerialNumber" of "DeviceDiscoveryResult" is not integer value.')



# This file is generated. Do not modify by hand.
from enum import Enum


class TriggerCondition(Enum):
    """
    Comparison operator for trigger condition.
    """

    EQ = 0
    """Equal To (==)"""

    NE = 1
    """Not Equal To (!=)"""

    GT = 2
    """Greater Than (>)"""

    GE = 3
    """Greater Than or Equal To (>=)"""

    LT = 4
    """Less Than (<)"""

    LE = 5
    """Less Than or Equal To (<=)"""



# This file is generated. Do not modify by hand.
# pylint: disable=line-too-long, unused-argument, f-string-without-interpolation, too-many-branches, too-many-statements, unnecessary-pass, unused-variable
from dataclasses import dataclass
from typing import Any, Dict, Optional
import decimal
import zaber_bson


@dataclass
class UnitConversionDescriptor:
    """
    Information about unit conversion.
    """

    dimension: str
    """
    Name of the dimension being converted.
    """

    conversion_function: str
    """
    Internal name of conversion function used.
    """

    scale: float
    """
    Scale factor used in conversion, if applicable.
    """

    resolution: Optional[int] = None
    """
    Microstep resolution used in conversion, if applicable.
    """

    @staticmethod
    def zero_values() -> 'UnitConversionDescriptor':
        return UnitConversionDescriptor(
            dimension="",
            conversion_function="",
            scale=0,
            resolution=None,
        )

    @staticmethod
    def from_binary(data_bytes: bytes) -> 'UnitConversionDescriptor':
        """" Deserialize a binary representation of this class. """
        data = zaber_bson.loads(data_bytes)  # type: Dict[str, Any]
        return UnitConversionDescriptor.from_dict(data)

    def to_binary(self) -> bytes:
        """" Serialize this class to a binary representation. """
        self.validate()
        return zaber_bson.dumps(self.to_dict())  # type: ignore

    def to_dict(self) -> Dict[str, Any]:
        return {
            'dimension': str(self.dimension or ''),
            'conversionFunction': str(self.conversion_function or ''),
            'scale': float(self.scale),
            'resolution': int(self.resolution) if self.resolution is not None else None,
        }

    @staticmethod
    def from_dict(data: Dict[str, Any]) -> 'UnitConversionDescriptor':
        return UnitConversionDescriptor(
            dimension=data.get('dimension'),  # type: ignore
            conversion_function=data.get('conversionFunction'),  # type: ignore
            scale=data.get('scale'),  # type: ignore
            resolution=data.get('resolution'),  # type: ignore
        )

    def validate(self) -> None:
        """" Validates the properties of the instance. """
        if self.dimension is not None:
            if not isinstance(self.dimension, str):
                raise ValueError(f'Property "Dimension" of "UnitConversionDescriptor" is not a string.')

        if self.conversion_function is not None:
            if not isinstance(self.conversion_function, str):
                raise ValueError(f'Property "ConversionFunction" of "UnitConversionDescriptor" is not a string.')

        if self.scale is None:
            raise ValueError(f'Property "Scale" of "UnitConversionDescriptor" is None.')

        if not isinstance(self.scale, (int, float, decimal.Decimal)):
            raise ValueError(f'Property "Scale" of "UnitConversionDescriptor" is not a number.')

        if self.resolution is not None:
            if not isinstance(self.resolution, (int, float, decimal.Decimal)):
                raise ValueError(f'Property "Resolution" of "UnitConversionDescriptor" is not a number.')

            if int(self.resolution) != self.resolution:
                raise ValueError(f'Property "Resolution" of "UnitConversionDescriptor" is not integer value.')



# This file is generated. Do not modify by hand.
from enum import Enum


class DigitalOutputAction(Enum):
    """
    Action type for digital output.
    """

    OFF = 0
    """Set the digital output low."""

    ON = 1
    """Set the digital output high."""

    TOGGLE = 2
    """Toggle the current state of the digital output."""

    KEEP = 3
    """Leave the digital output in its current state."""



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import List, Optional
from .call import call, call_async, call_sync
from .dto import requests as dto
from .dto.device_discovery_result import DeviceDiscoveryResult


class Tools:
    """
    Class providing various utility functions.
    """

    @staticmethod
    def list_serial_ports() -> List[str]:
        """
        Lists all serial ports on the computer.

        Returns:
            Array of serial port names.
        """
        request = dto.EmptyRequest(
        )
        response = call(
            "tools/list_serial_ports",
            request,
            dto.ToolsListSerialPortsResponse.from_binary)
        return response.ports

    @staticmethod
    async def list_serial_ports_async() -> List[str]:
        """
        Lists all serial ports on the computer.

        Returns:
            Array of serial port names.
        """
        request = dto.EmptyRequest(
        )
        response = await call_async(
            "tools/list_serial_ports",
            request,
            dto.ToolsListSerialPortsResponse.from_binary)
        return response.ports

    @staticmethod
    def get_message_router_pipe_path() -> str:
        """
        Returns path of message router named pipe on Windows
        or file path of unix domain socket on UNIX.

        Returns:
            Path of message router's named pipe or unix domain socket.
        """
        request = dto.EmptyRequest(
        )
        response = call_sync(
            "tools/get_message_router_pipe",
            request,
            dto.StringResponse.from_binary)
        return response.value

    @staticmethod
    def get_db_service_pipe_path() -> str:
        """
        Returns the path for communicating with a local device database service.
        This will be a named pipe on Windows and the file path of a unix domain socket on UNIX.

        Returns:
            Path of database service's named pipe or unix domain socket.
        """
        request = dto.EmptyRequest(
        )
        response = call_sync(
            "tools/get_db_service_pipe",
            request,
            dto.StringResponse.from_binary)
        return response.value

    @staticmethod
    def discover_tcp_devices(
            duration: int = 3000,
            interface_ip_address: Optional[str] = None
    ) -> List[DeviceDiscoveryResult]:
        """
        Discover Zaber devices shared over local network over TCP/IP.

        Args:
            duration: Optional time in ms to wait for mDNS discovery response (defaults to 3000).
            interface_ip_address: Specify which network interface to use by IP address.
                If no value or an empty string is provided, mDNS query is sent to all compatible network interfaces.

        Returns:
            Array of discovered devices.
        """
        request = dto.DiscoverMdnsRequest(
            duration=duration,
            interface_ip_address=interface_ip_address,
        )
        response = call(
            "tools/discover_tcp_devices",
            request,
            dto.DiscoverTCPDevicesResponse.from_binary)
        return response.result

    @staticmethod
    async def discover_tcp_devices_async(
            duration: int = 3000,
            interface_ip_address: Optional[str] = None
    ) -> List[DeviceDiscoveryResult]:
        """
        Discover Zaber devices shared over local network over TCP/IP.

        Args:
            duration: Optional time in ms to wait for mDNS discovery response (defaults to 3000).
            interface_ip_address: Specify which network interface to use by IP address.
                If no value or an empty string is provided, mDNS query is sent to all compatible network interfaces.

        Returns:
            Array of discovered devices.
        """
        request = dto.DiscoverMdnsRequest(
            duration=duration,
            interface_ip_address=interface_ip_address,
        )
        response = await call_async(
            "tools/discover_tcp_devices",
            request,
            dto.DiscoverTCPDevicesResponse.from_binary)
        return response.result
